﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;

namespace RadTreeView_Filter
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static string[] TeamsA = { "Barcelona", "Man United", "Chelsea", "Juventus" };
        static string[] TeamsB = { "Real Madrid", "Liverpool", "Arsenal", "Milan" };
        Random r;
        MainViewModel model;

        public MainWindow()
        {
            InitializeComponent();
            r = new Random();
            BindTreeView();
        }
      

        private void BindTreeView()
        {
            model = new MainViewModel();
            ItemModel rootModel = new ItemModel() { Text = "FootBall Matches"};
            ItemModel uefaClModel = new ItemModel() { Text = "Uefa CL" };
            ItemModel uefaLeagueModel = new ItemModel() { Text = "Uefa League Europe" };

            for (int i = 0; i < 4; i++)
            {
                ItemModel group = new ItemModel() { Text = "Group " + (i + 1) };
                for (int j = 0; j < 4; j++)
                {
                    ItemModel match = new ItemModel() { Text = TeamsA[r.Next(0,3)] + " : " + TeamsB[r.Next(0,3)]};
                    group.Children.Add(match);
                }
                uefaClModel.Children.Add(group);
            }
            ItemModel finals = new ItemModel() { Text = "Finals "  };
            ItemModel quarterFinals = new ItemModel { Text = "Quarter Finals" };
            ItemModel semiFinals = new ItemModel { Text = "Semi Finals" };
            ItemModel final = new ItemModel { Text = "Final" };
            for (int k = 0; k < 4; k++)
            {
                ItemModel match1 = new ItemModel() { Text = TeamsA[r.Next(0,3)] + " : " + TeamsB[r.Next(0,3)]};
                    quarterFinals.Children.Add(match1);
            }
            for (int p = 0; p < 2; p++)
            {
                ItemModel match2 = new ItemModel() { Text = TeamsA[r.Next(0,3)] + " : " + TeamsB[r.Next(0,3)]};
                    semiFinals.Children.Add(match2);
            }
            ItemModel match4 = new ItemModel() { Text = TeamsA[r.Next(0, 3)] + " : " + TeamsB[r.Next(0, 3)] };
            final.Children.Add(match4);
            finals.Children.Add(quarterFinals);
            finals.Children.Add(semiFinals);
            finals.Children.Add(final);
            uefaClModel.Children.Add(finals);

            for (int i = 0; i < 4; i++)
            {
                ItemModel group = new ItemModel(){ Text = "Group" + (i + 1) };
                for (int j = 0; j < 4; j++)
                {
                    ItemModel match3 = new ItemModel() { Text = TeamsA[r.Next(0, 3)] + " : " + TeamsB[r.Next(0, 3)] };
                    group.Children.Add(match3);                    
                }
                uefaLeagueModel.Children.Add(group);
            }

            rootModel.Children.Add(uefaClModel);
            rootModel.Children.Add(uefaLeagueModel);
            model.RootLevelNodes.Add(rootModel);

            this.treeView.ItemsSource = model.RootLevelNodes;
            this.treeView.ExpandAll();
        }

        private void searchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
           string searchtext = (sender as TextBox).Text;
           MarkModelsForFiltering_BFS(searchtext);
           this.treeView.ItemsSource = CreateFilteredModels(model.RootLevelNodes);
           this.treeView.ExpandAll();
        }

        private ObservableCollection<ItemModel> CreateFilteredModels( ObservableCollection<ItemModel> rootCollection)
        {
            ObservableCollection<ItemModel> filteredModel = new ObservableCollection<ItemModel>();
            foreach (var item in rootCollection)
            {
                if (item.IsRemovedByFilter == false)
                {
                    ItemModel newItem = new ItemModel() { Text = item.Text };
                    filteredModel.Add(newItem);
                    if (item.Children.Count > 0)
                    {
                        newItem.Children = CreateFilteredModels(item.Children);
                    }
                }
            }
            return filteredModel;
        }

        private void MarkModelsForFiltering_BFS(string filterText)
        {           
            Queue<ItemModel> queue = new Queue<ItemModel>();
            foreach (var item in model.RootLevelNodes)
	        {
		        queue.Enqueue(item);
	        }
            while(queue.Count>0)
            {
                ItemModel current = queue.Dequeue();
                current.IsRemovedByFilter = false;
                if (current.Text.ToUpper().Contains(filterText.ToUpper()))
                {
                    current.IsRemovedByFilter = false;
                    SetParentFilterStatus(current);
                }
                else
                {
                    current.IsRemovedByFilter = true;
                }                
               
                if (current.Children.Count > 0)
                {
                    foreach (var item in current.Children)
	                {
                        queue.Enqueue(item);
	                }                 
                }
            }
        }

        private void SetParentFilterStatus(ItemModel item)
        {
            ItemModel currentModel = item;
            while (currentModel.parentModel != null)
            {
                currentModel.parentModel.IsRemovedByFilter = false;
                currentModel = currentModel.parentModel;
            }
        }




    }
}
